#version 400

in vec2 texcoord;
out vec4 out_color;

uniform sampler2D heightfield;
uniform sampler2D sourceHeightfield;
uniform isampler2D watermap;
uniform isampler2D watergainmap;
uniform isampler2D flowmap;

const float terrain_size = 512.0;
const float terrain_height = 80.0;

// Compute normal from heightmap
vec3 compute_normal(vec2 terrain_coord) {
    vec3 pixel_size = vec3(1.0 / terrain_size, -1.0 / terrain_size, 0);
    float h_u0 = texture(heightfield, terrain_coord + pixel_size.yz).x * terrain_height;
    float h_u1 = texture(heightfield, terrain_coord + pixel_size.xz).x * terrain_height;
    
    float h_v0 = texture(heightfield, terrain_coord + pixel_size.zy).x * terrain_height;
    float h_v1 = texture(heightfield, terrain_coord + pixel_size.zx).x * terrain_height;

    vec3 va = normalize(vec3(1, 0.0, h_u1 - h_u0));
    vec3 vb = normalize(vec3(0.0, 1, h_v1 - h_v0));

    return normalize(cross(va, vb));
}



void main() {

    vec3 normal = compute_normal(texcoord);
    float lighting = max(0, dot(normal, normalize(vec3(0.6,0.4,1.0))));
    float height = texture(heightfield, texcoord).x;
    float heightSource = texture(sourceHeightfield, texcoord).x;
    float flow = texture(flowmap, texcoord).x * 0.00000001;
    float water = intBitsToFloat(texture(watermap, texcoord).x) * 10.0;
    float waterGain = float(texture(watergainmap, texcoord).x) / 0xFFFFFF * 10.0;

    vec3 diffuse = vec3(0.5);
    diffuse += vec3(0.2, 0.6, 1.0) * water;

    // diffuse = vec3(flow);

    diffuse *= lighting;

    out_color = vec4(diffuse, 1.0);
}