


from panda3d.core import *

loadPrcFileData("", "gl-debug #t")
loadPrcFileData("", "notify-level-glgsg fatal")

import direct.directbase.DirectStart
from PlaneTools import generateChunk



chunk = generateChunk(mesh_subdivisions=256, mesh_scale=2.0)
chunk.reparentTo(render)
chunk.setShader(Shader.load(Shader.SLGLSL, "Shader/terrain_vertex.glsl", "Shader/terrain_fragment.glsl"))

sourceHeightmap = loader.loadTexture("source2.png")
sourceHeightmap.setWrapU(SamplerState.WMClamp)
sourceHeightmap.setWrapV(SamplerState.WMClamp)
# sourceHeightmap.setFormat(Texture.FR16)
# sourceHeightmap.setFormat(Texture.FRgb8)

terrainSize = sourceHeightmap.getXSize()



base.accept("f3", base.toggleWireframe)
base.cam.setPos(600, 0, 800)
base.cam.lookAt(0, 0, 100)
base.disableMouse()

# EROSION

waterMap = Texture("WaterMap")
waterMap.setup2dTexture(terrainSize, terrainSize, Texture.TInt, Texture.FR32i)
waterMap.setClearColor(Vec4(0))
waterMap.clearImage()

waterGainMap = Texture("WaterGainMap")
waterGainMap.setup2dTexture(terrainSize, terrainSize, Texture.TInt, Texture.FR32i)
waterGainMap.setClearColor(Vec4(0))


flowMap = Texture("FlowMap")
flowMap.setup2dTexture(terrainSize, terrainSize, Texture.TInt, Texture.FR32i)
flowMap.setClearColor(Vec4(0))

flowMap.clearImage()

tempHeight = Texture("TempHeight")
tempHeight.setup2dTexture(terrainSize, terrainSize, Texture.TInt, Texture.FR32i)


def executeShader(src, params):
    shader = Shader.loadCompute(Shader.SLGLSL, src)
    dummy = NodePath("dummy")
    dummy.setShader(shader)

    for key, val in params.iteritems():
        dummy.setShaderInput(key, val)

    sattr = dummy.getAttrib(ShaderAttrib)
    base.graphicsEngine.dispatch_compute((terrainSize / 8, terrainSize / 8, 1), sattr, base.win.get_gsg())


# Copy the heightmap so we can modify it
executeShader("Shader/copy_height.compute", {"fromTex": sourceHeightmap, "destTex": tempHeight})

def erosionStep():
    global steps
    print "step"

    waterGainMap.clearImage()
    # executeShader("Shader/add_water.compute", {"watermap": waterMap})

    executeShader("Shader/movement.compute", {"watermap": waterMap, "heightmap": tempHeight, "watergainmap": waterGainMap, "flowmap": flowMap})
    executeShader("Shader/apply_gain.compute", {"watermap": waterMap, "watergainmap": waterGainMap})
    executeShader("Shader/evaporation.compute", {"watermap": waterMap, "heightmap": tempHeight})

    # Finally copy the heightmap back
    # executeShader("Shader/convert_height.compute", {"fromTex": tempHeight, "destTex": sourceHeightmap})

def multipleStep():
    for i in xrange(20):
        erosionStep()

def addWater():
    executeShader("Shader/add_water.compute", {"watermap": waterMap})
    executeShader("Shader/erosion.compute", {"watermap": waterMap, "heightmap": tempHeight})

addWater()

base.accept("w", addWater)
base.accept("w-repeat", addWater)

base.accept("t", erosionStep)
base.accept("t-repeat", erosionStep)

base.accept("z", multipleStep)
base.accept("z-repeat", multipleStep)



chunk.setShaderInput("heightfield", tempHeight)
chunk.setShaderInput("sourceHeightfield", sourceHeightmap)
chunk.setShaderInput("flowmap", flowMap)
chunk.setShaderInput("watermap", waterMap)
chunk.setShaderInput("watergainmap", waterGainMap)


run()